#include "stdafx.h"
#include "NetworkRemocon.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


class CSendStringInfo {
public:
	CNetworkRemocon *m_pRemocon;
	char *m_pBuffer;
	int m_Length;
	CNetworkRemoconReciver *m_pReciver;
	CSendStringInfo(CNetworkRemocon *pRemocon,const char *pBuffer,int Length,
					CNetworkRemoconReciver *pReciver) {
		m_pRemocon=pRemocon;
		m_pBuffer=new char[Length+1];
		CopyMemory(m_pBuffer,pBuffer,Length);
		m_pBuffer[Length]='\0';
		m_Length=Length;
		m_pReciver=pReciver;
	}
	~CSendStringInfo() {
		delete [] m_pBuffer;
	}
};


class CChannelMap {
	struct Map {
		int From;
		int To;
	};
public:
	CChannelMap();
	~CChannelMap();
	bool Add(int Form,int To);
};




CNetworkRemocon::CNetworkRemocon()
{
	m_fInitialized=false;
	m_hThread=NULL;
	m_Socket=INVALID_SOCKET;
	m_fConnected=false;
}


CNetworkRemocon::~CNetworkRemocon()
{
	if (m_fInitialized) {
		Shutdown();
		WSACleanup();
	}
}


bool CNetworkRemocon::Init(LPCSTR pszAddress,WORD Port)
{
	int Err;

	if (!m_fInitialized) {
		Err=WSAStartup(MAKEWORD(2,0),&m_WSAData);
		if (Err!=0)
			return false;
		m_fInitialized=true;
	} else
		Shutdown();
	m_Address=inet_addr(pszAddress);
	if (m_Address==INADDR_NONE)
		return false;
	m_Port=Port;
	return true;
}


bool CNetworkRemocon::Shutdown()
{
	if (m_fInitialized) {
		if (m_hThread!=NULL) {
			if (WaitForSingleObject(m_hThread,5000)==WAIT_TIMEOUT)
				TerminateThread(m_hThread,FALSE);
			CloseHandle(m_hThread);
			m_hThread=NULL;
		}
		if (m_Socket!=INVALID_SOCKET) {
			shutdown(m_Socket,1);
			closesocket(m_Socket);
			m_Socket=INVALID_SOCKET;
		}
	}
	return true;
}


DWORD CNetworkRemocon::SendProc(LPVOID pParam)
{
	CSendStringInfo *pInfo=(CSendStringInfo*)pParam;
	int Result;
	char Buffer[1024];

	if (pInfo->m_pRemocon->m_Socket==INVALID_SOCKET) {
		pInfo->m_pRemocon->m_Socket=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
		if (pInfo->m_pRemocon->m_Socket==INVALID_SOCKET)
			return FALSE;
	}
	if (!pInfo->m_pRemocon->m_fConnected) {
		struct sockaddr_in sockaddri;

		sockaddri.sin_family=AF_INET;
		sockaddri.sin_port=htons(pInfo->m_pRemocon->m_Port);
		sockaddri.sin_addr.S_un.S_addr=pInfo->m_pRemocon->m_Address;
		ZeroMemory(sockaddri.sin_zero,sizeof(sockaddri.sin_zero));
		Result=connect(pInfo->m_pRemocon->m_Socket,
							(struct sockaddr*)&sockaddri,sizeof(sockaddr_in));
		if (Result!=0) {
			delete pInfo;
			return FALSE;
		}
		pInfo->m_pRemocon->m_fConnected=true;
	}
	Result=send(pInfo->m_pRemocon->m_Socket,pInfo->m_pBuffer,pInfo->m_Length,0);
	if (Result==SOCKET_ERROR) {
		closesocket(pInfo->m_pRemocon->m_Socket);
		pInfo->m_pRemocon->m_Socket=INVALID_SOCKET;
		pInfo->m_pRemocon->m_fConnected=false;
		return FALSE;
	}
	Result=recv(pInfo->m_pRemocon->m_Socket,Buffer,sizeof(Buffer)-1,0);
	if (Result!=SOCKET_ERROR && pInfo->m_pReciver!=NULL) {
		Buffer[Result]='\0';
		pInfo->m_pReciver->OnRecive(Buffer);
	}
	delete pInfo;
	if (Result==SOCKET_ERROR || Result==0) {
		closesocket(pInfo->m_pRemocon->m_Socket);
		pInfo->m_pRemocon->m_Socket=INVALID_SOCKET;
		pInfo->m_pRemocon->m_fConnected=false;
		return Result==SOCKET_ERROR;
	}
	return TRUE;
}


bool CNetworkRemocon::Send(const char *pBuffer,int Length,
											CNetworkRemoconReciver *pReciver)
{
	DWORD ThreadID;
	CSendStringInfo *pInfo;

	if (!m_fInitialized)
		return false;
	if (m_hThread!=NULL) {
		if (WaitForSingleObject(m_hThread,0)==WAIT_TIMEOUT)
			return false;
		CloseHandle(m_hThread);
	}
	pInfo=new CSendStringInfo(this,pBuffer,Length,pReciver);
	m_hThread=CreateThread(NULL,0,SendProc,pInfo,0,&ThreadID);
	if (m_hThread==NULL) {
		delete pInfo;
		return false;
	}
	return true;
}


bool CNetworkRemocon::SetChannel(int ChannelNo)
{
	char szText[16];
	int Length;

	if (m_ChannelList.NumChannels()>0) {
		int i;

		for (i=0;i<m_ChannelList.NumChannels();i++) {
			if (m_ChannelList.GetChannelNo(i)==ChannelNo+1) {
				Length=wsprintfA(szText,"SetCh:%d",i);
				return Send(szText,Length);
			}
		}
	} else {
		Length=wsprintfA(szText,"SetCh:%d",ChannelNo);
		return Send(szText,Length);
	}
	return false;
}


class CGetChannelReciver : public CNetworkRemoconReciver {
public:
	CNetworkRemoconReciver *m_pReciver;
	void OnRecive(LPCSTR pszText);
};

void CGetChannelReciver::OnRecive(LPCSTR pszText)
{
	LPCSTR p;
	char szChannel[16];
	int i;

	p=pszText;
	while (*p!='\t') {
		if (*p=='\0')
			return;
		p++;
	}
	p++;
	for (i=0;*p>='0' && *p<='9';i++) {
		if (i==sizeof(szChannel))
			return;
		szChannel[i]=*p++;
	}
	if (i==0)
		return;
	szChannel[i]='\0';
	m_pReciver->OnRecive(szChannel);
}


bool CNetworkRemocon::GetChannel(CNetworkRemoconReciver *pReciver)
{
	static CGetChannelReciver Reciver;

	Reciver.m_pReciver=pReciver;
	return Send("GetChList",9,&Reciver);
}


bool CNetworkRemocon::LoadChannelText(LPCTSTR pszFileName,
											const CChannelList *pChannelList)
{
	HANDLE hFile;
	DWORD FileSize,Read;
	char *pszText,*p;

	hFile=CreateFile(pszFileName,GENERIC_READ,FILE_SHARE_READ,NULL,
														OPEN_EXISTING,0,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return false;
	FileSize=GetFileSize(hFile,NULL);
	if (FileSize==INVALID_FILE_SIZE || FileSize==0) {
		CloseHandle(hFile);
		return false;
	}
	pszText=new char[FileSize+1];
	if (!ReadFile(hFile,pszText,FileSize,&Read,NULL) || Read!=FileSize) {
		CloseHandle(hFile);
		return false;
	}
	pszText[FileSize]='\0';
	CloseHandle(hFile);
	m_ChannelList.Clear();
	p=pszText;
	while (*p!='\0') {
		char *pszName;
		int Space,Channel;

		while (*p=='\r' || *p=='\n')
			p++;
		if (*p==';') {	// Comment
			while (*p!='\r' && *p!='\n' && *p!='\0')
				p++;
			continue;
		}
		pszName=p;
		while (*p!='\t' && *p!='\0')
			p++;
		if (*p=='\0')
			break;
		*p++='\0';
		Space=0;
		while (*p>='0' && *p<='9') {
			Space=Space*10+(*p-'0');
			p++;
		}
		if (*p!='\t')
			break;
		p++;
		Channel=0;
		while (*p>='0' && *p<='9') {
			Channel=Channel*10+(*p-'0');
			p++;
		}
		int i;
		for (i=0;i<pChannelList->NumChannels();i++) {
			if (pChannelList->GetSpace(i)==Space
					&& pChannelList->GetChannelIndex(i)==Channel) {
				m_ChannelList.AddChannel(*pChannelList->GetChannelInfo(i));
				break;
			}
		}
		if (i==pChannelList->NumChannels()) {
			TCHAR szName[MAX_CHANNEL_NAME];

			MultiByteToWideChar(CP_ACP,0,pszName,-1,szName,MAX_CHANNEL_NAME);
			m_ChannelList.AddChannel(Space,0,Channel,
				pChannelList->NumChannels()==0?
							m_ChannelList.NumChannels()+1:Channel+1,0,szName);
		}
		while (*p!='\r' && *p!='\n' && *p!='\0')
			p++;
	}
	delete [] pszText;
	return true;
}
